<?php

namespace Accounting\Invoice\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Accounting\Invoice\Forms\Invoice as InvoiceForm;
use Accounting\Invoice\Http\Requests\InvoiceRequest;
use Accounting\Invoice\Interfaces\InvoiceRepositoryInterface;
use Accounting\Invoice\Repositories\Eloquent\Filters\InvoiceResourceFilter;
use Accounting\Invoice\Repositories\Eloquent\Presenters\InvoiceListPresenter;

/**
 * Resource controller class for invoice.
 */
class InvoiceResourceController extends BaseController
{

    /**
     * Initialize invoice resource controller.
     *
     *
     * @return null
     */
    public function __construct(InvoiceRepositoryInterface $invoice)
    {
        parent::__construct();
        $this->form = InvoiceForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('accounting.invoice.modules'), 'invoice', guard_url('invoice'));
        $this->repository = $invoice;
    }

    /**
     * Display a list of invoice.
     *
     * @return Response
     */
    public function index(InvoiceRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(InvoiceResourceFilter::class)
            ->setPresenter(InvoiceListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('invoice::invoice.names'))
            ->view('invoice::invoice.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display invoice.
     *
     * @param Request $request
     * @param Model   $invoice
     *
     * @return Response
     */
    public function show(InvoiceRequest $request, InvoiceRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('invoice::invoice.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('invoice::invoice.show')
            ->output();
    }

    /**
     * Show the form for creating a new invoice.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(InvoiceRequest $request, InvoiceRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('invoice::invoice.name'))
            ->view('invoice::invoice.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new invoice.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(InvoiceRequest $request, InvoiceRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('invoice::invoice.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('invoice/invoice/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/invoice/invoice'))
                ->redirect();
        }

    }

    /**
     * Show invoice for editing.
     *
     * @param Request $request
     * @param Model   $invoice
     *
     * @return Response
     */
    public function edit(InvoiceRequest $request, InvoiceRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('invoice::invoice.name'))
            ->view('invoice::invoice.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the invoice.
     *
     * @param Request $request
     * @param Model   $invoice
     *
     * @return Response
     */
    public function update(InvoiceRequest $request, InvoiceRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('invoice::invoice.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('invoice/invoice/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('invoice/invoice/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the invoice.
     *
     * @param Model   $invoice
     *
     * @return Response
     */
    public function destroy(InvoiceRequest $request, InvoiceRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('invoice::invoice.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('invoice/invoice/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('invoice/invoice/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
